@@ -45,6 +45,7 @@
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 
+#include "gnome-settings-profile.h"
 #include "gsd-mouse-manager.h"
 
 #include "gsd-locate-pointer.h"
@@ -439,8 +440,6 @@ filter (GdkXEvent *xevent,
                         XUngrabKeyboard (gdk_x11_get_default_xdisplay (),
                                          xev->xkey.time);
                 }
-
-                return GDK_FILTER_REMOVE;
         }
         return GDK_FILTER_CONTINUE;
 }
@@ -552,7 +551,7 @@ set_mousetweaks_daemon (GsdMouseManager *manager,
         GError *error = NULL;
         gchar *comm;
 
-        comm = g_strdup_printf ("mousetweaks %s", 
+        comm = g_strdup_printf ("mousetweaks %s",
                                 (dwell_enable || delay_enable) ? "" : "-s");
 
         if (! g_spawn_command_line_async (comm, &error)) {
@@ -651,6 +650,7 @@ gsd_mouse_manager_start (GsdMouseManager *manager,
         GConfClient *client;
 
         g_debug ("Starting mouse manager");
+        gnome_settings_profile_start (NULL);
 
         client = gconf_client_get_default ();
 
@@ -664,6 +664,8 @@ gsd_mouse_manager_start (GsdMouseManager *manager,
 
         g_object_unref (client);
 
+        gnome_settings_profile_end (NULL);
+
         return TRUE;
 }
 
