@@ -48,14 +48,15 @@
 #include "libsounds/sound-properties.h"
 
 #include "gsd-sound-manager.h"
+#include "gnome-settings-profile.h"
 
 #define GSD_SOUND_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_SOUND_MANAGER, GsdSoundManagerPrivate))
 
 struct GsdSoundManagerPrivate
 {
-        gboolean padding;
         /* esd/PulseAudio pid */
-        GPid pid;
+        GPid  pid;
+        guint child_watch_id;
 };
 
 enum {
@@ -71,55 +72,124 @@ G_DEFINE_TYPE (GsdSoundManager, gsd_sound_manager, G_TYPE_OBJECT)
 static gpointer manager_object = NULL;
 
 static void
-reset_esd_pid (GPid pid, gint status, gpointer user_data)
+reset_esd_pid (GPid             pid,
+               int              status,
+               GsdSoundManager *manager)
 {
-	GsdSoundManager *manager = (GsdSoundManager *) user_data;
+        g_debug ("GsdSoundManager: **** child (pid:%d) done (%s:%d)",
+                 (int) pid,
+                 WIFEXITED (status) ? "status"
+                 : WIFSIGNALED (status) ? "signal"
+                 : "unknown",
+                 WIFEXITED (status) ? WEXITSTATUS (status)
+                 : WIFSIGNALED (status) ? WTERMSIG (status)
+                 : -1);
 
-	if (pid == manager->priv->pid)
-		manager->priv->pid = 0;
+        if (pid == manager->priv->pid) {
+                manager->priv->pid = 0;
+        }
+
+        g_spawn_close_pid (manager->priv->pid);
+        manager->priv->child_watch_id = 0;
 }
 
-/* start_gnome_sound
- *
- * Start GNOME sound.
- */
 static gboolean
 start_gnome_sound (GsdSoundManager *manager)
 {
-	char  *argv[] = { "esd", "-nobeeps", NULL};
-	GError *err = NULL;
-	time_t  starttime;
+        char    *argv[] = { "esd", "-nobeeps", NULL};
+        GError  *error;
+        gboolean res;
+        time_t   starttime;
+        gboolean ret;
 
-	if (!g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL,
-			    &manager->priv->pid, &err)) {
-		g_printerr ("Could not start esd: %s\n", err->message);
-		g_error_free (err);
-		return FALSE;
-	}
+        ret = FALSE;
 
-	g_child_watch_add (manager->priv->pid, reset_esd_pid, NULL);
+        gnome_settings_profile_start (NULL);
 
-	starttime = time (NULL);
-	gnome_sound_init (NULL);
+        error = NULL;
+        res = g_spawn_async (NULL,
+                             argv,
+                             NULL,
+                             G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
+                             NULL,
+                             NULL,
+                             &manager->priv->pid,
+                             &error);
+        if (! res) {
+                g_printerr ("Could not start esd: %s\n", error->message);
+                g_error_free (error);
+                goto out;
+        }
 
-	while (gnome_sound_connection_get () < 0
-	       && ((time (NULL) - starttime) < 4))
-	{
-		g_usleep (200);
-		gnome_sound_init (NULL);
-	}
+        manager->priv->child_watch_id = g_child_watch_add (manager->priv->pid,
+                                                           (GChildWatchFunc)reset_esd_pid,
+                                                           manager);
 
-	return gnome_sound_connection_get () >= 0;
+        starttime = time (NULL);
+        gnome_sound_init (NULL);
+
+        while (gnome_sound_connection_get () < 0
+               && ((time (NULL) - starttime) < 4)) {
+                g_usleep (200);
+                gnome_sound_init (NULL);
+        }
+
+        ret = gnome_sound_connection_get () >= 0;
+
+ out:
+        gnome_settings_profile_end (NULL);
+
+        return ret;
+}
+
+static int
+wait_on_child (int pid)
+{
+        int status;
+
+ wait_again:
+        if (waitpid (pid, &status, 0) < 0) {
+                if (errno == EINTR) {
+                        goto wait_again;
+                } else if (errno == ECHILD) {
+                        ; /* do nothing, child already reaped */
+                } else {
+                        g_debug ("GsdSoundManager: waitpid () should not fail");
+                }
+        }
+
+        return status;
+}
+
+static void
+stop_child (GsdSoundManager *manager)
+{
+        int exit_status;
+
+        if (manager->priv->pid <= 0) {
+                return;
+        }
+
+        if (manager->priv->child_watch_id > 0) {
+                g_source_remove (manager->priv->child_watch_id);
+                manager->priv->child_watch_id = 0;
+        }
+
+        if (kill (manager->priv->pid, SIGTERM) == -1) {
+                g_printerr ("Failed to kill esd (pid %d)\n", manager->priv->pid);
+                return;
+        }
+
+        exit_status = wait_on_child (manager->priv->pid);
+
+        g_spawn_close_pid (manager->priv->pid);
+        manager->priv->pid = -1;
 }
 
 #ifdef HAVE_ESD
 static gboolean set_esd_standby = TRUE;
 #endif
 
-/* stop_gnome_sound
- *
- * Stop GNOME sound.
- */
 static void
 stop_gnome_sound (GsdSoundManager *manager)
 {
@@ -130,12 +200,7 @@ stop_gnome_sound (GsdSoundManager *manager)
 #else
         gnome_sound_shutdown ();
 
-	if (manager->priv->pid) {
-		if (kill (manager->priv->pid, SIGTERM) == -1)
-			g_printerr ("Failed to kill esd (pid %d)\n", manager->priv->pid);
-		else
-			manager->priv->pid = 0;
-	}
+        stop_child (manager);
 #endif
 }
 
@@ -143,7 +208,6 @@ struct reload_foreach_closure {
         gboolean enable_system_sounds;
 };
 
-
 /* reload_foreach_cb
  *
  * For a given SoundEvent, reload the sound file associate with the event.
@@ -158,6 +222,8 @@ reload_foreach_cb (SoundEvent *event, gpointer data)
 
         closure = data;
 
+        gnome_settings_profile_start (NULL);
+
         key = sound_event_compose_key (event);
 
 #ifdef HAVE_ESD
@@ -165,24 +231,28 @@ reload_foreach_cb (SoundEvent *event, gpointer data)
          * esd allows multiple samples with the same name,
          * putting memory to waste. */
         sid = esd_sample_getid (gnome_sound_connection_get (), key);
-        if (sid >= 0)
+        if (sid >= 0) {
                 esd_sample_free (gnome_sound_connection_get (), sid);
+        }
 #endif
         /* We only disable sounds for system events.  Other events, like sounds
          * in games, should be preserved.  The games should have their own
          * configuration for sound anyway.
          */
         if ((strcmp (event->category, "gnome-2") == 0
-             || strcmp (event->category, "gtk-events-2") == 0))
+             || strcmp (event->category, "gtk-events-2") == 0)) {
                 do_load = closure->enable_system_sounds;
-        else
+        } else {
                 do_load = TRUE;
+        }
 
-        if (!do_load)
+        if (!do_load) {
                 goto out;
+        }
 
-        if (!event->file || !strcmp (event->file, ""))
+        if (!event->file || !strcmp (event->file, "")) {
                 goto out;
+        }
 
         if (event->file[0] == '/') {
                 file = g_strdup (event->file);
@@ -192,18 +262,22 @@ reload_foreach_cb (SoundEvent *event, gpointer data)
                                                   event->file, TRUE, NULL);
         }
 
-        if (!file)
+        if (!file) {
                 goto out;
+        }
 
         sid = gnome_sound_sample_load (key, file);
 
-        if (sid < 0)
+        if (sid < 0) {
                 g_warning (_("Couldn't load sound file %s as sample %s"),
                            file, key);
+        }
 
         g_free (file);
 
  out:
+        gnome_settings_profile_end (NULL);
+
         g_free (key);
 }
 
@@ -218,13 +292,11 @@ apply_settings (GsdSoundManager *manager)
         gboolean        event_sounds;
         struct reload_foreach_closure closure;
 
+        gnome_settings_profile_start (NULL);
+
         client = gconf_client_get_default ();
 
-#ifdef HAVE_ESD
         enable_sound = gconf_client_get_bool (client, "/desktop/gnome/sound/enable_esd", NULL);
-#else
-        enable_sound = TRUE;
-#endif
         event_sounds = gconf_client_get_bool (client, "/desktop/gnome/sound/event_sounds", NULL);
         /* FIXME this is completely bogus, the entry doesn't exist */
         event_changed_new = gconf_client_get_int  (client, "/desktop/gnome/sound/event_changed", NULL);
@@ -233,8 +305,9 @@ apply_settings (GsdSoundManager *manager)
 
         if (enable_sound) {
                 if (gnome_sound_connection_get () < 0) {
-                        if (!start_gnome_sound (manager))
-                            return;
+                        if (!start_gnome_sound (manager)) {
+                                goto out;
+                        }
                 }
 #ifdef HAVE_ESD
                 else if (set_esd_standby) {
@@ -244,7 +317,7 @@ apply_settings (GsdSoundManager *manager)
 #endif
         } else {
 #ifdef HAVE_ESD
-                if (!set_esd_standby)
+                if (! set_esd_standby)
 #endif
                         stop_gnome_sound (manager);
         }
@@ -261,6 +334,9 @@ apply_settings (GsdSoundManager *manager)
                 sound_properties_foreach (props, reload_foreach_cb, &closure);
                 gtk_object_destroy (GTK_OBJECT (props));
         }
+ out:
+        gnome_settings_profile_end (NULL);
+
 }
 
 static void
@@ -293,11 +369,15 @@ gsd_sound_manager_start (GsdSoundManager *manager,
 {
         g_debug ("Starting sound manager");
 
+        gnome_settings_profile_start (NULL);
+
         register_config_callback (manager,
                                   "/desktop/gnome/sound",
                                   (GConfClientNotifyFunc)sound_callback);
         apply_settings (manager);
 
+        gnome_settings_profile_end (NULL);
+
         return TRUE;
 }
 
@@ -363,9 +443,11 @@ gsd_sound_manager_constructor (GType                  type,
 static void
 gsd_sound_manager_dispose (GObject *object)
 {
-        GsdSoundManager *sound_manager;
+        GsdSoundManager *manager;
 
-        sound_manager = GSD_SOUND_MANAGER (object);
+        manager = GSD_SOUND_MANAGER (object);
+
+        stop_child (manager);
 
         G_OBJECT_CLASS (gsd_sound_manager_parent_class)->dispose (object);
 }
